'use strict';

var fs = require('fs');
var fse = require('fs-extra');
var path = require('path');

var appDirectory = fs.realpathSync(process.cwd());
function resolveApp(relativePath) {
  return path.resolve(appDirectory, relativePath);
}

var appNodeModulesDirectory = resolveApp('../');

function getFileStat(absolutePath) {
  try {
    return fs.statSync(absolutePath);
  } catch (e) {
    if (e.code === 'ENOENT') return null;
    throw e;
  }
}

function getConfig(webpackEnv, type) {
  if (!type) return path.resolve(appNodeModulesDirectory, 'react-scripts/config/webpack.config.' + webpackEnv + '.js');
  if (type === 'my') return resolveApp('./config/webpack.config.' + webpackEnv + '.js');

  return path.resolve(appNodeModulesDirectory, 'react-scripts/config/webpack.config.' + webpackEnv + '.' + type + '.js');
}

function extendConfig(webpackEnv, callback) {
  if (webpackEnv !== 'dev' && webpackEnv !== 'prod') {
    console.error('Only dev and prod is supported');
    return;
  }

  return moveAsOrigin(webpackEnv).then(function () {
    return createWebpackConfig(webpackEnv);
  }).then(function () {
    return callback();
  });
}

function moveAsOrigin(webpackEnv) {
  var webpackConfig = getConfig(webpackEnv);
  var webpackConfigOrigin = getConfig(webpackEnv, 'origin');
  var originStat = getFileStat(webpackConfigOrigin);

  if (!originStat) {
    return fse.move(webpackConfig, webpackConfigOrigin);
  } else {
    return Promise.resolve();
  }
}

function createWebpackConfig(webpackEnv) {
  var webpackConfig = getConfig(webpackEnv);
  var stat = getFileStat(webpackConfig);
  if (!stat) {
    return fse.outputFile(webpackConfig, 'const config = require(\'./webpack.config.' + webpackEnv + '.origin\')\nconst customConfigFunc = require(\'../../../config/webpack.config.' + webpackEnv + '\')\n\nmodule.exports = customConfigFunc(config)  \n    ');
  }
}

module.exports = extendConfig;